# -*- coding: utf-8 -*-
import xbmc, xbmcaddon
from sys import argv
try: from urlparse import parse_qsl
except ImportError: from urllib.parse import parse_qsl
from modules.utils import selection_dialog, multiselect_dialog
from modules.utils import local_string as ls
# from modules.utils import logger

__addon__ = xbmcaddon.Addon(id='plugin.video.fen')

def similar_recommendations_choice():
	import json
	from modules.nav_utils import build_url
	from modules.utils import selection_dialog
	import tikimeta
	params = dict(parse_qsl(argv[2].replace('?','')))
	db_type = params.get('db_type')
	meta_type = 'movie' if db_type  =='movies' else 'tvshow'
	try: meta_user_info = json.loads(params.get('meta_user_info'))
	except: meta_user_info = tikimeta.retrieve_user_info()
	dl = [ls(32592), ls(32593)]
	fl = ['trakt_%s_related' % db_type, 'tmdb_%s_recommendations' % db_type]
	mode = 'build_%s_list' % meta_type
	choice = selection_dialog(dl, fl)
	if not choice: return
	try:
		sim_recom_params = {'mode': mode, 'action': choice, 'sim_recom_tmdb': params.get('sim_recom_tmdb'), 'sim_recom_imdb': params.get('sim_recom_imdb'), 'sim_recom_name': params.get('sim_recom_name'), 'from_search': params.get('from_search')}
		xbmc.executebuiltin('Container.Update(%s)' % build_url(sim_recom_params))
	except: return

def movie_reviews(tmdb_id, rootname, poster):
	from apis.tmdb_api import tmdb_movies_reviews
	reviews_info = tmdb_movies_reviews(tmdb_id)
	total_results = reviews_info['total_results']
	if total_results == 0:
		from modules.nav_utils import notification
		return notification(ls(32760), 3500)
	import xbmcgui
	dialog = xbmcgui.Dialog()
	review_list = []
	all_reviews = reviews_info['results']
	for item in all_reviews:
		line1 = item['author']
		line2 = '[I]%s...[/I]' % item['content'][:60]
		icon = poster
		listitem = xbmcgui.ListItem(line1, line2)
		listitem.setArt({'icon': icon})
		listitem.setProperty('review_id', str(item['id']))
		listitem.setProperty('reviewer_name', item['author'])
		listitem.setProperty('review_content', item['content'])
		review_list.append(listitem)
	selection = dialog.select('Fen', review_list, useDetails=True)
	if selection >= 0:
		review_id = review_list[selection].getProperty('review_id')
		reviewer_name = review_list[selection].getProperty('reviewer_name')
		review_content = review_list[selection].getProperty('review_content')
	else: return
	heading = '%s - [B]%s[/B]' % (rootname, reviewer_name)
	dialog.textviewer(heading, review_content)
	if total_results > 1: return movie_reviews(tmdb_id, rootname, poster)

def set_quality(quality_setting=None):
	params = dict(parse_qsl(argv[2].replace('?','')))
	quality_setting = quality_setting if quality_setting else params['quality_setting']
	include = ls(32188)
	setting = 'autoplay_quality' if quality_setting == 'autoplay' else 'results_quality'
	dl = ['%s SD' % include, '%s 720p' % include, '%s 1080p' % include, '%s 4K' % include]
	fl = ['SD', '720p', '1080p', '4K']
	try: preselect = [fl.index(i) for i in __addon__.getSetting(setting).split(', ')]
	except: preselect = []
	filters = multiselect_dialog('Fen', dl, fl, preselect)
	if filters is None: return
	if filters == []:
		import xbmcgui
		xbmcgui.Dialog().ok('Fen', '', ls(32574))
		return set_quality(quality_setting)
	from modules.nav_utils import toggle_setting
	toggle_setting(setting, ', '.join(filters))

def enable_scrapers():
	from modules.settings import active_scrapers
	from modules.nav_utils import toggle_setting
	scrapers = ['external', 'furk', 'easynews', 'rd-cloud', 'pm-cloud', 'ad-cloud', 'local', 'downloads', 'folders']
	scraper_names = [ls(32118).upper(), ls(32069).upper(), ls(32070).upper(), ls(32098).upper(), ls(32097).upper(), ls(32099).upper(), ls(32100).upper(), ls(32107).upper(), ls(32108).upper()]
	preselect = [scrapers.index(i) for i in active_scrapers(group_folders=True)]
	scraper_choice = multiselect_dialog('Fen', scraper_names, scrapers, preselect=preselect)
	if scraper_choice is None: return
	return [toggle_setting('provider.%s' % i, ('true' if i in scraper_choice else 'false')) for i in scrapers]

def set_subtitle_action():
	from modules.nav_utils import toggle_setting
	choices = ((ls(32192), '0'), (ls(32193), '1'), (ls(32027), '2'))
	choice = selection_dialog([i[0] for i in choices], [i[1] for i in choices])
	if choice: return toggle_setting('subtitles.subs_action', choice)

def set_display_mode():
	from modules.nav_utils import toggle_setting
	names = (ls(32235), ls(32236), ls(32237))
	settings = ('0', '1', '2')
	choice = selection_dialog(names, settings, 'Fen')
	if choice: return toggle_setting('display_mode', choice)

def results_sorting_choice():
	from modules.nav_utils import toggle_setting
	quality = ls(32241)
	provider = ls(32583)
	size = ls(32584)
	choices = [('%s, %s, %s' % (quality, provider, size), '0'), ('%s, %s, %s' % (quality, size, provider), '1'), ('%s, %s, %s' % (provider, quality, size), '2'),
			   ('%s, %s, %s' % (provider, size, quality), '3'), ('%s, %s, %s' % (size, quality, provider), '4'), ('%s, %s, %s' % (size, provider, quality), '5')]
	choice = selection_dialog([i[0] for i in choices], choices, 'Fen')
	if choice:
		toggle_setting('results.sort_order_display', choice[0])
		toggle_setting('results.sort_order', choice[1])

def subscriptions_update_interval():
	from modules.nav_utils import toggle_setting
	import time
	from datetime import datetime, timedelta
	from modules.nav_utils import open_settings
	params = dict(parse_qsl(argv[2].replace('?','')))
	subscription_timer = __addon__.getSetting('subscription_timer')
	if params['update_type'] == 'interval':
		subscriptions_update_label = 'subscriptions_update_label1'
		hours = ls(32594)
		intervals = [('2 %s' % hours, 2), ('4 %s' % hours, 4), ('8 %s' % hours, 8), ('12 %s' % hours, 12), ('16 %s' % hours, 16), ('24 %s' % hours, 24)]
		try: last_run_time = datetime.strptime(__addon__.getSetting('service_time'), "%Y-%m-%d %H:%M:%S") - timedelta(hours=int(subscription_timer))
		except TypeError: last_run_time = datetime(*(time.strptime(__addon__.getSetting('service_time'), "%Y-%m-%d %H:%M:%S")[0:6])) - timedelta(hours=int(subscription_timer))
		choice = selection_dialog([i[0] for i in intervals], intervals)
		if not choice: choice = ('24 Hours', 24)
		interval = str(choice[1])
		interval_display = choice[0]
		new_run_time = last_run_time + timedelta(hours=int(interval))
		new_run_time = new_run_time.strftime("%Y-%m-%d %H:%M:%S")
	else:
		subscriptions_update_label = 'subscriptions_update_label2'
		intervals = []
		for i in range(24): intervals.append(('%02d:00' % i, i))
		choice = selection_dialog([i[0] for i in intervals], intervals, 'Fen')
		if not choice: choice = ('06:00', 6)
		sub_hour = choice[1]
		interval_display = choice[0]
		interval = subscription_timer
		now = datetime.today()
		sub_day = now.day + 1 if sub_hour == 0 else now.day
		new_run_time = now.replace(day=sub_day, hour=sub_hour, minute=0, second=0, microsecond=0) + timedelta(days=1)
		difference = new_run_time - now
		if '1 day' in str(difference): new_run_time = now.replace(day=sub_day, hour=sub_hour, minute=0, second=0, microsecond=0)
		new_run_time = new_run_time.strftime("%Y-%m-%d %H:%M:%S")
	toggle_setting('subscription_timer', interval)
	toggle_setting(subscriptions_update_label, interval_display)
	toggle_setting('service_time', new_run_time)

def unaired_episode_color_choice():
	from modules.nav_utils import toggle_setting
	chosen_color = color_chooser('Fen', no_color=True)
	if chosen_color: toggle_setting('unaired_episode_colour', chosen_color)

def scraper_dialog_color_choice():
	from modules.nav_utils import toggle_setting
	params = dict(parse_qsl(argv[2].replace('?','')))
	setting ='int_dialog_highlight' if params['setting'] == 'internal' else 'ext_dialog_highlight'
	chosen_color = color_chooser('Fen')
	if chosen_color: toggle_setting(setting, chosen_color)

def next_episode_color_choice(setting=None):
	from modules.nav_utils import open_settings
	params = dict(parse_qsl(argv[2].replace('?','')))
	choices = [(ls(32078), 'nextep.airdate_colour'),
				(ls(32079), 'nextep.unaired_colour'),
				(ls(32080), 'nextep.unwatched_colour')]
	prelim_setting = params.get('setting', None) if not setting else setting
	title, setting = [(i[0], i[1]) for i in choices if i[0] == prelim_setting][0]
	chosen_color = color_chooser('Fen - %s' % title, no_color=True)
	if chosen_color: __addon__.setSetting(setting, chosen_color)

def next_episode_options_choice(setting=None):
	from modules.nav_utils import notification
	from modules.utils import selection_dialog
	params = dict(parse_qsl(argv[2].replace('?','')))
	off = ls(32027).upper()
	on = ls(32090).upper()
	choices = [
			('Sort Type', 'nextep.sort_type', [(ls(32226).upper(), '0'), (ls(32227).upper(), '1'), (ls(32228).upper(), '2')]),
			('Sort Order', 'nextep.sort_order', [(ls(32225).upper(), '0'), (ls(32224).upper(), '1')]),
			('Include Unaired', 'nextep.include_unaired', [(off, 'false'), (on, 'true')]),
			('Include Trakt or Fen Unwatched', 'nextep.include_unwatched', [(off, 'false'), (on, 'true')]),
			('Cache To Disk', 'nextep.cache_to_disk', [(off, 'false'), (on, 'true')]),
			('Include Airdate in Title', 'nextep.include_airdate', [(off, 'false'), (on, 'true')]),
			('Airdate Format', 'nextep.airdate_format', [(ls(32229).upper(), '0'), (ls(32230).upper(), '1'), (ls(32231).upper(), '2')])
				]
	prelim_setting = params.get('setting') if not setting else setting
	title, setting = [(i[0], i[1]) for i in choices if i[0] == prelim_setting][0]
	full_list = [i[2] for i in choices if i[0] == prelim_setting][0]
	dialog_list = [i[0] for i in full_list]
	function_list = [i[1] for i in full_list]
	selection = selection_dialog(dialog_list, function_list, 'Fen - %s' % title)
	if not selection: return
	setting_name = [i[0] for i in full_list if i[1] == selection][0]
	__addon__.setSetting(setting, selection)
	notification(ls(32576), 6000)

def next_episode_context_choice():
	from modules.utils import selection_dialog
	from modules.nav_utils import toggle_setting, build_url
	from modules import settings
	params = dict(parse_qsl(argv[2].replace('?','')))
	content_settings = settings.nextep_content_settings()
	display_settings = settings.nextep_display_settings()
	airdate_replacement = [('%d-%m-%Y', ls(32229)), ('%Y-%m-%d', ls(32230)), ('%m-%d-%Y', ls(32231))]
	sort_type_status = (ls(32226), ls(32227), ls(32228))[content_settings['sort_type']]
	sort_order_status = (ls(32224), ls(32225))[content_settings['sort_order']]
	toggle_sort_order_SETTING = ('nextep.sort_order', ('0' if sort_order_status == ls(32224) else '1'))
	cache_to_disk_status = str(content_settings['cache_to_disk'])
	toggle_cache_to_disk_SETTING = ('nextep.cache_to_disk', ('true' if cache_to_disk_status == 'False' else 'false'))
	unaired_status = str(content_settings['include_unaired'])
	toggle_unaired_SETTING = ('nextep.include_unaired', ('true' if unaired_status == 'False' else 'false'))
	unwatched_status = str(content_settings['include_unwatched'])
	toggle_unwatched_SETTING = ('nextep.include_unwatched', ('true' if unwatched_status == 'False' else 'false'))
	airdate_status = str(display_settings['include_airdate'])
	toggle_airdate_SETTING = ('nextep.include_airdate', ('true' if airdate_status == 'False' else 'false'))
	airdate_format = settings.nextep_airdate_format()
	airdate_format_status = [airdate_format.replace(i[0], i[1]) for i in airdate_replacement if i[0] == airdate_format][0]
	airdate_highlight = display_settings['airdate_colour'].capitalize()
	unaired_highlight = display_settings['unaired_colour'].capitalize()
	unwatched_highlight = display_settings['unwatched_colour'].capitalize()
	base_string = '%s: [I]%s [B]%s[/B][/I]' % ('%s', ls(32598), '%s')
	choices = [
			(ls(32596).upper(), 'manage_in_progress'),
			(base_string % (ls(32066).upper(), sort_type_status), 'Sort Type'),
			(base_string % (ls(32067), sort_order_status), 'toggle_sort_order'),
			(base_string % (ls(32075), cache_to_disk_status), 'toggle_cache_to_disk'),
			(base_string % (ls(32073), unaired_status), 'toggle_unaired'),
			(base_string % (ls(32074), unwatched_status), 'toggle_unwatched'),
			(base_string % (ls(32076), airdate_status), 'toggle_airdate'),
			(base_string % (ls(32077), airdate_format_status), 'Airdate Format'),
			(base_string % (ls(32078), airdate_highlight), 'Airdate'),
			(base_string % (ls(32079), unaired_highlight), 'Unaired'),
			(base_string % (ls(32080), unwatched_highlight), 'Unwatched')]
	if settings.watched_indicators() == 0: choices.append((ls(32597), 'manage_unwatched'))
	if settings.watched_indicators() in (1,2): choices.append((ls(32497) % ls(32037), 'clear_cache'))
	string = 'Fen - %s' % ls(32599)
	dialog_list = [i[0] for i in choices]
	function_list = [i[1] for i in choices]
	choice = selection_dialog(dialog_list, function_list, string)
	if not choice: return
	if choice in ('toggle_sort_order', 'toggle_cache_to_disk', 'toggle_unaired', 'toggle_unwatched', 'toggle_airdate'):
		setting = eval(choice + '_SETTING')
		toggle_setting(setting[0], setting[1])
	elif choice == 'clear_cache':
		from modules.nav_utils import clear_cache
		clear_cache('trakt')
	else:
		if choice in ('manage_in_progress', 'manage_unwatched'):
			xbmc.executebuiltin('Container.Update(%s)' % build_url({'mode': 'build_next_episode_manager', 'action': choice})); return
		elif choice in ('Airdate','Unaired', 'Unwatched'): function = next_episode_color_choice
		else: function = next_episode_options_choice
		function(choice)
	xbmc.executebuiltin("Container.Refresh")
	xbmc.executebuiltin('RunPlugin(%s)' % build_url({'mode': 'next_episode_context_choice'}))

def scraper_quality_color_choice():
	from modules.nav_utils import toggle_setting
	params = dict(parse_qsl(argv[2].replace('?','')))
	setting = params['setting']
	chosen_color = color_chooser('Fen')
	if chosen_color: toggle_setting(setting, chosen_color)

def external_scrapers_manager():
	import xbmcgui
	import json
	from modules.nav_utils import build_url
	dialog = xbmcgui.Dialog()
	icon = xbmcaddon.Addon(id='script.module.openscrapers').getAddonInfo('icon')
	fail_color = 'crimson'
	all_color = 'mediumvioletred'
	debrid_color = __addon__.getSetting('prem.identify')
	torrent_color = __addon__.getSetting('torrent.identify')
	enable_string, disable_string, specific_string, all_string, failing_string, ext_scrapers_string, reset_string, stats_string, scrapers_string, debrid_string, torrent_string = ls(32055), ls(32024), ls(32536), ls(32525), ls(32529), ls(32118), ls(32531), ls(32532), ls(32533), ls(32534), ls(32535)
	failing_scrapers_string, all_scrapers_string, debrid_scrapers_string = '%s %s' % (failing_string, scrapers_string), '%s %s' % (all_string, scrapers_string), '%s %s' % (debrid_string, scrapers_string)
	torrent_scrapers_string, fail1_string, fail2_string = '%s %s' % (torrent_string, scrapers_string), '%s %s %s' % (disable_string, failing_string, ext_scrapers_string), '%s %s %s %s' % (reset_string, failing_string, ext_scrapers_string, stats_string)
	enable_string_base, disable_string_base, enable_disable_string_base = '%s %s %s %s' % (enable_string, all_string, '%s', scrapers_string), '%s %s %s %s' % (disable_string, all_string, '%s', scrapers_string), '%s/%s %s %s %s' % (enable_string, disable_string, specific_string, '%s', scrapers_string)
	failure_base = '[COLOR %s][B]%s : [/B][/COLOR]' % (fail_color, failing_scrapers_string.upper())
	all_scrapers_base = '[COLOR %s][B]%s : [/B][/COLOR]' % (all_color, all_scrapers_string.upper())
	debrid_scrapers_base = '[COLOR %s][B]%s : [/B][/COLOR]' % (debrid_color, debrid_scrapers_string.upper())
	torrent_scrapers_base = '[COLOR %s][B]%s : [/B][/COLOR]' % (torrent_color, torrent_scrapers_string.upper())
	tools_menu = \
		[(failure_base, fail1_string, {'mode': 'external_scrapers_disable'}),
		(failure_base, fail2_string, {'mode': 'external_scrapers_reset_stats'}),
		(all_scrapers_base, enable_string_base % '', {'mode': 'toggle_all', 'folder': 'all_eng', 'setting': 'true'}),
		(all_scrapers_base, disable_string_base % '', {'mode': 'toggle_all', 'folder': 'all_eng', 'setting': 'false'}),
		(all_scrapers_base, enable_disable_string_base % '',{'mode': 'enable_disable_specific_all', 'folder': 'all_eng'}),
		(debrid_scrapers_base, enable_string_base % debrid_string, {'mode': 'toggle_all', 'folder': 'en_DebridOnly', 'setting': 'true'}),
		(debrid_scrapers_base, disable_string_base % debrid_string, {'mode': 'toggle_all', 'folder': 'en_DebridOnly', 'setting': 'false'}),
		(debrid_scrapers_base, enable_disable_string_base % debrid_string, {'mode': 'enable_disable_specific_all', 'folder': 'en_DebridOnly'}),
		(torrent_scrapers_base, enable_string_base % torrent_string, {'mode': 'toggle_all', 'folder': 'en_Torrent', 'setting': 'true'}),
		(torrent_scrapers_base, disable_string_base % torrent_string, {'mode': 'toggle_all', 'folder': 'en_Torrent', 'setting': 'false'}),
		(torrent_scrapers_base, enable_disable_string_base % torrent_string, {'mode': 'enable_disable_specific_all', 'folder': 'en_Torrent'})]
	choice_list = []
	for item in tools_menu:
		line1 = item[0]
		line2 = '[B][I]%s[/I][/B]' % item[1]
		listitem = xbmcgui.ListItem(line1, line2)
		listitem.setArt({'icon': icon})
		choice_list.append(listitem)
	chosen_tool = dialog.select("Fen", choice_list, useDetails=True)
	if chosen_tool < 0: return
	from modules import external_source_utils
	params = tools_menu[chosen_tool][2]
	mode = params['mode']
	if mode == 'external_scrapers_disable':
		external_source_utils.external_scrapers_disable()
	elif mode == 'external_scrapers_reset_stats':
		external_source_utils.external_scrapers_reset_stats()
	elif mode == 'toggle_all':
		external_source_utils.toggle_all(params['folder'], params['setting'])
	elif mode == 'enable_disable_specific_all':
		external_source_utils.enable_disable_specific_all(params['folder'])
	return external_scrapers_manager()

def scraper_color_choice():
	from modules.nav_utils import toggle_setting
	params = dict(parse_qsl(argv[2].replace('?','')))
	choices = [('furk', 'provider.furk_colour'),
				('easynews', 'provider.easynews_colour'),
				('pm-cloud', 'provider.pm-cloud_colour'),
				('rd-cloud', 'provider.rd-cloud_colour'),
				('ad-cloud', 'provider.ad-cloud_colour'),
				('local', 'provider.local_colour'),
				('downloads', 'provider.downloads_colour'),
				('folders', 'provider.folders_colour'),
				('premium', 'prem.identify'),
				('torrent', 'torrent.identify'),
				('second_line', 'secondline.identify')]
	setting = [i[1] for i in choices if i[0] == params.get('setting')][0]
	dialog = 'Fen'
	chosen_color = color_chooser(dialog, no_color=True)
	if chosen_color: toggle_setting(setting, chosen_color)

def folder_sources_choice():
	import xbmcgui
	from modules.nav_utils import toggle_setting
	params = dict(parse_qsl(argv[2].replace('?','')))
	setting = params['setting']
	folder = xbmcgui.Dialog().browse(0, 'Fen', '')
	if not folder: folder = 'None'
	__addon__.setSetting(setting, folder)

def internal_scrapers_order_choice():
	import xbmcgui
	from modules.settings import internal_scraper_order, internal_scrapers_order_display
	from modules.nav_utils import toggle_setting
	window = xbmcgui.Window(10000)
	display = []
	label_list = [('files', ls(32493), '$ADDON[plugin.video.fen 32493]'), ('furk', ls(32069), '$ADDON[plugin.video.fen 32069]'), 
				  ('easyews', ls(32070), '$ADDON[plugin.video.fen 32070]'), ('cloud', ls(32586), '$ADDON[plugin.video.fen 32586]')]
	try: current_setting = window.getProperty('FEN_internal_scrapers_order').split(', ')
	except: current_setting = ['']
	try: current_display_setting = window.getProperty('FEN_internal_scrapers_order_display').split(', ')
	except: current_display_setting = ['']
	if len(current_setting) != 4 or len(current_display_setting) != 4:
		current_setting = internal_scraper_order()
		current_display_setting = internal_scrapers_order_display()
	for i in current_display_setting:
		for x in label_list:
			if i == x[2]: display.append(x[1])
	adjust_scraper = selection_dialog(display, [(i, current_display_setting[current_setting.index(i)]) for i in current_setting], 'Fen')
	if not adjust_scraper:
		window.clearProperty('FEN_internal_scrapers_order')
		window.clearProperty('FEN_internal_scrapers_order_display')
		return
	str_insert = [i[1] for i in label_list if i[2] == adjust_scraper[1]][0]
	choices = [(ls(32588) % str_insert, (0, current_setting.index(adjust_scraper[0]))),
			   (ls(32589) % str_insert, (1, current_setting.index(adjust_scraper[0]))),
			   (ls(32590) % str_insert, (2, current_setting.index(adjust_scraper[0]))),
			   (ls(32591) % str_insert, (3, current_setting.index(adjust_scraper[0])))]
	positioning_info = selection_dialog([i[0] for i in choices], [i[1] for i in choices], 'Fen - %s' % (ls(32587) % adjust_scraper[1]))
	if not positioning_info: return internal_scrapers_order_choice()
	new_position = positioning_info[0]
	current_position = positioning_info[1]
	current_setting.insert(new_position, current_setting.pop(current_position))
	current_display_setting.insert(new_position, current_display_setting.pop(current_position))
	new_order_setting = (', ').join(current_setting)
	new_order_setting_display = (', ').join(current_display_setting)
	toggle_setting('results.internal_scrapers_order', new_order_setting)
	xbmc.sleep(100)
	toggle_setting('results.internal_scrapers_order_display', new_order_setting_display)
	window.setProperty('FEN_internal_scrapers_order', new_order_setting)
	window.setProperty('FEN_internal_scrapers_order_display', new_order_setting_display)
	xbmc.sleep(200)
	return internal_scrapers_order_choice()

def build_add_to_remove_from_list(meta='', media_type='', orig_mode='', from_search=''):
	import json
	from modules.settings import trakt_list_subscriptions, watched_indicators
	from modules.nav_utils import build_url
	params = dict(parse_qsl(argv[2].replace('?','')))
	media_type = params.get('media_type', '')
	orig_mode = params.get('orig_mode', '')
	from_search = params.get('from_search', '')
	meta = json.loads(params.get('meta', None))
	add_to_str = ls(32602)
	remove_from_str = ls(32603)
	main_listing = [('%s...' % add_to_str, 'add'), ('%s...' % remove_from_str, 'remove')]
	mlc = selection_dialog([i[0] for i in main_listing], [i[1] for i in main_listing], 'Fen')
	if mlc == None: return
	heading = '%s ' % add_to_str if mlc == 'add' else '%s ' % remove_from_str
	listing = [(heading + ls(32205), 'trakt'), (heading + ls(32453), 'favourites')]
	if not trakt_list_subscriptions(): listing.append((heading + ls(32194), 'subscriptions'))
	if media_type == 'tvshow' and watched_indicators() == 0: listing.append((heading + ls(32483), 'unwatched_next_episode'))
	if mlc == 'remove': listing.append((heading + ls(32604) % (ls(32028) if media_type == 'movie' else ls(32029)), 'refresh'))
	choice = selection_dialog([i[0] for i in listing], [i[1] for i in listing], 'Fen')
	if choice == None: return
	elif choice == 'trakt': url = {'mode': ('trakt.trakt_add_to_list' if mlc == 'add' else 'trakt.trakt_remove_from_list'), 'tmdb_id': meta["tmdb_id"], 'imdb_id': meta["imdb_id"], 'tvdb_id': meta["tvdb_id"], 'db_type': media_type}
	elif choice == 'favourites': url = {'mode': ('add_to_favourites' if mlc == 'add' else 'remove_from_favourites'), 'db_type': media_type, 'tmdb_id': meta["tmdb_id"], 'title': meta['title']}
	elif choice == 'subscriptions': url = {'mode': 'subscriptions_add_remove', 'action': mlc, 'db_type': media_type, 'tmdb_id': meta["tmdb_id"], 'orig_mode': orig_mode}
	elif choice == 'unwatched_next_episode': url = {'mode': 'add_next_episode_unwatched', 'action': mlc, 'title': meta["title"], 'tmdb_id': meta["tmdb_id"], 'imdb_id': meta["imdb_id"]}
	elif choice == 'refresh': url = {'mode': 'refresh_cached_data', 'db_type': media_type, 'id_type': 'tmdb_id', 'media_id': meta['tmdb_id']}
	xbmc.executebuiltin('RunPlugin(%s)' % build_url(url))

def extras_menu(params):
	import json
	from modules.utils import merge_dicts
	from modules.settings import addon_installed
	from modules.nav_utils import build_url, movie_genres, tvshow_genres, networks
	def _process_dicts(_key, _dict):
		new_dict = {}
		for key, value in _dict.items():
			if key in _key: new_dict[key] = value
		return new_dict
	def process_list_dicts(_key, _value, _return_value, dict_list):
		for item in dict_list:
			if _key in item and item[_key] == _value:
				return item[_return_value]
	meta = json.loads(params['meta'])
	media_type = params['media_type']
	rootname = meta['rootname']
	tmdb_id = meta['tmdb_id']
	trailer = meta['trailer']
	director = meta.get('director', None)
	year = meta.get('year', None)
	genre = meta.get('genre', None)
	network = meta.get('studio', None)
	extra_info = meta.get('extra_info', None)
	if media_type == 'movies' and extra_info:
		collection_name = extra_info.get('collection_name', None)
		collection_id = extra_info.get('collection_id', None)
	else:
		collection_name = None
		collection_id = None
	if media_type in ('tv', 'season', 'episode'):
		info_name = ls(32029)
		base_media = 'tv'
	else:
		info_name = ls(32028)
		base_media = 'movies'
	if network:
		if base_media == 'tv': network_id = process_list_dicts('name', network, 'id', networks)
		else: network_id = network
	else: network_id = None
	if genre: genre_dict = _process_dicts(genre, merge_dicts(movie_genres, tvshow_genres))
	else: genre_dict = None
	sim_recom_runner = "RunPlugin(%s)" % build_url({"mode": "similar_recommendations_choice", "db_type": base_media, 'sim_recom_name': rootname, "sim_recom_tmdb": tmdb_id, "sim_recom_imdb": meta['imdb_id'], "meta_user_info": params['meta_user_info']})
	if params['all_trailers'] and meta.get('all_trailers', False): trailer_runner = "RunPlugin(%s)" % build_url({'mode': 'play_trailer', 'url': trailer, 'all_trailers': json.dumps(meta['all_trailers'])})
	else: trailer_runner = "RunPlugin(%s)" % build_url({'mode': 'play_trailer', 'url': trailer})
	actors_runner = "RunPlugin(%s)" % build_url({"mode": "people_search.extras_show_all_actors", "media_rootname": rootname, "full_cast": json.dumps(meta['cast'])})
	director_runner = "RunPlugin(%s)" % build_url({"mode": "people_search.extras_person_data", "person_name": director})
	year_runner = "RunPlugin(%s)" % build_url({"mode": "container_update", "final_params": json.dumps({'mode': 'build_%s_list' % ('movie' if base_media == 'movies' else 'tvshow'), 'action': 'tmdb_%s_year' % base_media, 'year': year})})
	genre_runner = "RunPlugin(%s)" % build_url({"mode": "container_update", "final_params": json.dumps({'mode': 'build_%s_list' % ('movie' if base_media == 'movies' else 'tvshow'), 'action': 'tmdb_%s_genres' % base_media, 'genre_list': json.dumps(genre_dict)})})
	network_runner = "RunPlugin(%s)" % build_url({"mode": "container_update", "final_params": json.dumps({'mode': 'build_%s_list' % ('movie' if base_media == 'movies' else 'tvshow'), 'action': 'tmdb_%s_networks' % base_media, 'network_id': network_id, 'network_name': network})})
	collection_runner = "RunPlugin(%s)" % build_url({"mode": "container_update", "final_params": json.dumps({'mode': 'build_movie_list', 'action': 'tmdb_movies_collection', 'collection_id': collection_id})})
	reviews_runner = "RunPlugin(%s)" % build_url({'mode': 'movie_reviews_choice', 'rootname': rootname, 'tmdb_id': tmdb_id, 'poster': meta['poster']})
	extra_info_runner = "RunPlugin(%s)" % build_url({'mode': 'media_extra_info_choice', 'media_type': media_type, 'meta': json.dumps(meta), 'extra_info': json.dumps(extra_info), 'params': json.dumps(params)})
	images_posters_runner = "RunPlugin(%s)" % build_url({"mode": "container_update", "final_params": json.dumps({'mode': 'media_image_results', 'db_type': base_media, 'tmdb_id': tmdb_id, 'image_type': 'posters'})})
	images_backdrops_runner = "RunPlugin(%s)" % build_url({"mode": "container_update", "final_params": json.dumps({'mode': 'media_image_results', 'db_type': base_media, 'tmdb_id': tmdb_id, 'image_type': 'backdrops'})})
	play_random_params = {'mode': 'play_fetch_random', 'db_type': 'episode', 'meta': json.dumps(meta)}
	if media_type in ('season', 'episode'): play_random_params['default_season'] = params['default_season']
	play_random_runner = "RunPlugin(%s)" % build_url(play_random_params)
	extended_info_runner = 'RunScript(script.extendedinfo,info=%s,id=%s)' % (('extendedinfo' if base_media == 'movies' else 'extendedtvinfo'), tmdb_id)
	choices = []
	if extra_info: choices += [(ls(32605) % info_name, extra_info_runner)]
	choices += [("%s/%s" % (ls(32592), ls(32593)), sim_recom_runner)]
	if trailer: choices += [(ls(32606), trailer_runner)]
	if media_type == 'movies': choices += [(ls(32607), reviews_runner)]
	choices += [(ls(32608), actors_runner)]
	if media_type == 'movies' and director: choices += [(ls(32609), director_runner)]
	if addon_installed('script.extendedinfo'): choices += [(ls(32610), extended_info_runner)]
	if collection_id: choices += [(ls(32611) % collection_name, collection_runner)]
	if year: choices += [(ls(32612) % year, year_runner)]
	if genre_dict: choices += [(ls(32613) % genre, genre_runner)]
	if network_id: choices += [(ls(32614) % (ls(32615) if base_media == 'movies' else ls(32480), network), network_runner)]
	choices += [(ls(32616), images_posters_runner)]
	choices += [(ls(32617), images_backdrops_runner)]
	if media_type != 'movies': choices += [("%s%s" % (ls(32618), ' [B][I]%s %s[/I][/B]' % (ls(32537), str(params['default_season'])) if media_type in ('season', 'episode') else ''), play_random_runner)]
	choice = selection_dialog([i[0] for i in choices], [i[1] for i in choices], meta['title'])
	if not choice: return
	xbmc.executebuiltin(choice)

def options_menu(from_results=False, suggestion=None, list_name=None, play_params=None):
	from modules.nav_utils import toggle_setting, build_url, open_settings, clear_cache, cached_page_clear, clear_and_rescrape
	from modules import settings
	params = dict(parse_qsl(argv[2].replace('?','')))
	content = xbmc.getInfoLabel('Container.Content')
	if not content: content = params.get('content', None)
	from_results = params.get('from_results', from_results)
	suggestion = params.get('suggestion', suggestion)
	list_name = params.get('list_name', list_name)
	play_params = params.get('play_params', play_params)
	on_str = ls(32090)
	off_str = ls(32027)
	currently_str = ls(32598)
	open_str = ls(32641).upper()
	settings_str = ls(32247).upper()
	furk_easy_search_str = '%s/%s %s: [B][I]%s[/I][/B]' % (ls(32069).upper(), ls(32070).upper(), ls(32450).upper(), '%s')
	base_str = '%s%s: [I]%s [B]%s[/B][/I]' % ('%s', '%s', currently_str, '%s')
	autoplay_status, autoplay_toggle, filter_setting = (on_str, 'false', 'autoplay_quality') if settings.auto_play() else (off_str, 'true', 'results_quality')
	quality_filter_setting = 'autoplay' if autoplay_status == on_str else 'results'
	autoplay_next_status, autoplay_next_toggle = (on_str, 'false') if settings.autoplay_next_episode() else (off_str, 'true')
	display_mode = settings.display_mode()
	display_mode_status = ls(32235) if display_mode == 0 else ls(32236) if display_mode == 1 else ls(32237)
	results_sorting_status = __addon__.getSetting('results.sort_order_display')
	autoplay_hevc = settings.autoplay_hevc()
	autoplay_hevc_status = ls(32188) if autoplay_hevc == 0 else ls(32189) if autoplay_hevc == 1 else ls(32190)
	current_subs_action = __addon__.getSetting('subtitles.subs_action')
	current_subs_action_status = 'Auto' if current_subs_action == '0' else ls(32193) if current_subs_action == '1' else off_str
	active_scrapers = [i.replace('-', '') for i in settings.active_scrapers(group_folders=True)]
	current_scrapers_status = ', '.join([i.upper()[:3] for i in active_scrapers]) if len(active_scrapers) > 0 else 'N/A'
	current_filter_status =  ', '.join(settings.quality_filter(filter_setting))
	indicators_status, indicators_toggle = (ls(32037), '0') if settings.watched_indicators() in (1, 2) else (ls(32036), '1')
	cached_torrents_status, cached_torrents_toggle = (on_str, 'false') if __addon__.getSetting('torrent.check.cache') =='true' else (off_str, 'true')
	uncached_torrents_status, uncached_torrents_toggle = (on_str, 'false') if __addon__.getSetting('torrent.display.uncached') =='true' else (off_str, 'true')
	furk_easy_suggestion = params.get('suggestion', '')
	listing = []  
	if play_params: listing += [(ls(32637).upper(), 'rescrape_select')]
	listing += [(base_str % ('', ls(32175).upper(), autoplay_status), 'toggle_autoplay')]
	if autoplay_status == on_str:
		listing += [(base_str % (' - ', ls(32177).upper(), autoplay_hevc_status), 'set_autoplay_hevc')]
		listing += [(base_str % (' - ', ls(32178).upper(), autoplay_next_status), 'toggle_autoplay_next')]
	listing += [(base_str % ('', '%s %s' % (ls(32055).upper(), ls(32533).upper()), current_scrapers_status), 'enable_scrapers')]
	if autoplay_status == off_str:
		listing += [(base_str % ('', ls(32638).upper(), display_mode_status), 'set_display_mode')]
		listing += [(base_str % ('', ls(32151).upper(), results_sorting_status), 'set_results_sorting')]
	listing += [(base_str % ('', ls(32105).upper(), current_filter_status), 'set_filters')]
	listing += [(base_str % ('', ls(32183).upper(), current_subs_action_status), 'set_subs_action')]
	if settings.cache_page(): listing += [(ls(32639).upper(), 'clear_cache_page')]
	listing += [(base_str % ('', ls(32034).upper(), indicators_status), 'toggle_indicators')]
	if 'external' in active_scrapers:
		listing += [(base_str % ('', ls(32159).upper(), cached_torrents_status), 'toggle_cached_torrents')]
		if cached_torrents_status == on_str:
			listing += [(base_str % (' - ', ls(32160).upper(), uncached_torrents_status), 'toggle_torrents_display_uncached')]
	if content in ('movies', 'episodes', 'movie', 'episode'): listing += [(furk_easy_search_str % furk_easy_suggestion, 'search_directly')]
	if settings.watched_indicators() in (1,2): listing += [((ls(32497) % ls(32037)).upper(), 'clear_trakt_cache')]
	listing += [('%s %s' % (ls(32118).upper(), ls(32513).upper()), 'open_external_scrapers_manager')]
	listing += [('%s %s %s' % (open_str, ls(32523).upper(), settings_str), 'open_meta_settings')]
	listing += [('%s %s %s' % (open_str, ls(32522).upper(), settings_str), 'open_scraper_settings')]
	listing += [('%s %s %s' % (open_str, ls(32036).upper(), settings_str), 'open_fen_settings')]
	listing += [('[B]%s[/B]' % ls(32640).upper(), 'save_and_exit')]
	xbmc.sleep(500)
	choice = selection_dialog([i[0] for i in listing], [i[1] for i in listing])
	if choice == 'rescrape_select': return clear_and_rescrape(play_params)
	elif choice == 'toggle_autoplay': toggle_setting('auto_play', autoplay_toggle)
	elif choice == 'set_autoplay_hevc': set_autoplay_hevc()
	elif choice == 'toggle_autoplay_next': toggle_setting('autoplay_next_episode', autoplay_next_toggle)
	elif choice == 'enable_scrapers': enable_scrapers()
	elif choice == 'set_display_mode': set_display_mode()
	elif choice == 'set_results_sorting': results_sorting_choice()
	elif choice == 'set_filters': set_quality(quality_filter_setting)
	elif choice == 'set_subs_action': set_subtitle_action()
	elif choice == 'clear_cache_page': cached_page_clear(action=list_name)
	elif choice == 'toggle_indicators': toggle_setting('watched_indicators', indicators_toggle)
	elif choice == 'toggle_cached_torrents': toggle_setting('torrent.check.cache', cached_torrents_toggle)
	elif choice == 'toggle_torrents_display_uncached': toggle_setting('torrent.display.uncached', uncached_torrents_toggle)
	elif choice == 'search_directly': furk_easynews_direct_search_choice(suggestion, from_results, list_name)
	elif choice == 'clear_trakt_cache': clear_cache('trakt')
	elif choice == 'open_external_scrapers_manager': external_scrapers_manager()
	elif choice == 'open_meta_settings': xbmc.executebuiltin('Addon.OpenSettings(script.module.tikimeta)')
	elif choice == 'open_scraper_settings': xbmc.executebuiltin('Addon.OpenSettings(script.module.openscrapers)')
	elif choice == 'open_fen_settings': open_settings('0.0')
	if choice in ('clear_cache_page', 'toggle_indicators', 'clear_trakt_cache') and content in ('movies', 'tvshows', 'seasons', 'episodes'): xbmc.executebuiltin('Container.Refresh')
	if choice in (None, 'rescrape_select', 'save_and_exit', 'clear_cache_page', 'toggle_indicators', 'clear_trakt_cache', 'search_directly', 'open_meta_settings', 'open_scraper_settings', 'open_fen_settings', 'open_external_scrapers_manager'): return
	xbmc.executebuiltin('RunPlugin(%s)' % build_url({'mode': 'options_menu_choice', 'from_results': from_results, 'suggestion': suggestion, 'list_name': list_name, 'play_params': play_params}))

def media_extra_info(media_type, meta, extra_info, params):
	import xbmcgui
	import json
	params = json.loads(params)
	extra_info = json.loads(extra_info)
	meta = json.loads(meta)
	body = []
	tagline_str, premiered_str, rating_str, votes_str, runtime_str, genres_str, budget_str, revenue_str, director_str, writer_str = ls(32619), ls(32620), ls(32621), ls(32623), ls(32622), ls(32624), ls(32625), ls(32626), ls(32627), ls(32628)
	studio_str, collection_str, homepage_str, status_str, type_str, classification_str, network_str, created_by_str, last_aired_str, next_aired_str, seasons_str, episodes_str = ls(32615), ls(32499), ls(32629), ls(32630), ls(32631), ls(32632), ls(32480), ls(32633), ls(32634), ls(32635), ls(32636), ls(32506)
	if media_type == 'movies':
		body.append('[B]%s:[/B] %s' % (tagline_str, meta['tagline']))
		if 'status' in extra_info: body.append('[B]%s:[/B] %s' % (status_str, extra_info['status']))
		body.append('[B]%s:[/B] %s' % (premiered_str, meta['premiered']))
		body.append('[B]%s:[/B] %s (%s %s)' % (rating_str, meta['rating'], meta['votes'], votes_str))
		body.append('[B]%s:[/B] %s mins' % (runtime_str, str(meta['duration']/60)))
		body.append('[B]%s:[/B] %s' % (genres_str, meta['genre']))
		if 'budget' in extra_info: body.append('[B]%s:[/B] %s' % (budget_str, extra_info['budget']))
		if 'revenue' in extra_info: body.append('[B]%s:[/B] %s' % (revenue_str, extra_info['revenue']))
		body.append('[B]%s:[/B] %s' % (director_str, meta['director']))
		body.append('[B]%s:[/B] %s' % (writer_str, meta['writer']))
		body.append('[B]%s:[/B] %s' % (studio_str, meta['studio']))
		if 'collection_name' in extra_info: body.append('[B]%s:[/B] %s' % (collection_str, extra_info['collection_name']))
		if 'homepage' in extra_info: body.append('[B]%s:[/B] %s' % (homepage_str, extra_info['homepage']))
	else:
		if 'type' in extra_info: body.append('[B]%s:[/B] %s' % (type_str, extra_info['type']))
		if 'status' in extra_info: body.append('[B]%s:[/B] %s' % (status_str, extra_info['status']))
		body.append('[B]%s:[/B] %s' % (premiered_str, meta['premiered']))
		body.append('[B]%s:[/B] %s (%s %s)' % (rating_str, meta['rating'], meta['votes'], votes_str))
		body.append('[B]%s:[/B] %s mins' % (runtime_str, str(meta['duration']/60)))
		body.append('[B]%s:[/B] %s' % (classification_str, meta['mpaa']))
		body.append('[B]%s:[/B] %s' % (genres_str, meta['genre']))
		body.append('[B]%s:[/B] %s' % (network_str, meta['studio']))
		if 'created_by' in extra_info: body.append('[B]%s:[/B] %s' % (created_by_str, extra_info['created_by']))
		if 'last_episode_to_air' in extra_info: body.append('[B]%s:[/B] %s' % (last_aired_str, extra_info['last_episode_to_air']))
		if 'next_episode_to_air' in extra_info: body.append('[B]%s:[/B] %s' % (next_aired_str, extra_info['next_episode_to_air']))
		body.append('[B]%s:[/B] %s' % (seasons_str, meta['total_seasons']))
		body.append('[B]%s:[/B] %s' % (episodes_str, meta['total_episodes']))
		if 'homepage' in extra_info: body.append('[B]%s:[/B] %s' % (homepage_str, extra_info['homepage']))
	xbmcgui.Dialog().select(meta['title'], body)
	return extras_menu(params)

def set_autoplay_hevc():
	from modules.nav_utils import toggle_setting
	options = [(ls(32188), '0'), (ls(32189), '1'), (ls(32190), '2')]
	options_choice = selection_dialog([i[0] for i in options], [i[1] for i in options], 'Fen')
	return toggle_setting('autoplay_hevc', options_choice)

def furk_easynews_direct_search_choice(suggestion, from_results, list_name):
	from modules.nav_utils import build_url
	search = ls(32450).upper()
	direct_search_furk_params = {'mode': 'furk.search_furk', 'db_type': 'video', 'suggestion': suggestion}
	direct_search_easynews_params = {'mode': 'easynews.search_easynews', 'suggestion': suggestion}
	choices = [('%s %s' % (search, ls(32069).upper()), direct_search_furk_params), ('%s %s' % (search, ls(32070).upper()), direct_search_easynews_params)]
	choice = selection_dialog([i[0] for i in choices], [i[1] for i in choices])
	if not choice: xbmc.executebuiltin('RunPlugin(%s)' % build_url({'mode': 'playback_menu', 'from_results': from_results, 'suggestion': suggestion, 'list_name': list_name}))
	else: xbmc.executebuiltin('Container.Update(%s)' % build_url(choice))

def color_chooser(msg_dialog, no_color=False):
	color_chart = [
		  'black', 'white', 'whitesmoke', 'gainsboro', 'lightgray', 'silver', 'darkgray', 'gray', 'dimgray',
		  'snow', 'floralwhite', 'ivory', 'beige', 'cornsilk', 'antiquewhite', 'bisque', 'blanchedalmond',
		  'burlywood', 'darkgoldenrod', 'ghostwhite', 'azure', 'lightsaltegray', 'lightsteelblue',
		  'powderblue', 'lightblue', 'skyblue', 'lightskyblue', 'deepskyblue', 'dodgerblue', 'royalblue',
		  'blue', 'mediumblue', 'midnightblue', 'navy', 'darkblue', 'cornflowerblue', 'slateblue', 'slategray',
		  'yellowgreen', 'springgreen', 'seagreen', 'steelblue', 'teal', 'fuchsia', 'deeppink', 'darkmagenta',
		  'blueviolet', 'darkviolet', 'darkorchid', 'darkslateblue', 'darkslategray', 'indigo', 'cadetblue',
		  'darkcyan', 'darkturquoise', 'turquoise', 'cyan', 'paleturquoise', 'lightcyan', 'mintcream', 'honeydew',
		  'aqua', 'aquamarine', 'chartreuse', 'greenyellow', 'palegreen', 'lawngreen', 'lightgreen', 'lime',
		  'mediumspringgreen', 'mediumturquoise', 'lightseagreen', 'mediumaquamarine', 'mediumseagreen',
		  'limegreen', 'darkseagreen', 'forestgreen', 'green', 'darkgreen', 'darkolivegreen', 'olive', 'olivedab',
		  'darkkhaki', 'khaki', 'gold', 'goldenrod', 'lightyellow', 'lightgoldenrodyellow', 'lemonchiffon',
		  'yellow', 'seashell', 'lavenderblush', 'lavender', 'lightcoral', 'indianred', 'darksalmon',
		  'lightsalmon', 'pink', 'lightpink', 'hotpink', 'magenta', 'plum', 'violet', 'orchid', 'palevioletred',
		  'mediumvioletred', 'purple', 'maroon', 'mediumorchid', 'mediumpurple', 'mediumslateblue', 'thistle',
		  'linen', 'mistyrose', 'palegoldenrod', 'oldlace', 'papayawhip', 'moccasin', 'navajowhite', 'peachpuff',
		  'sandybrown', 'peru', 'chocolate', 'orange', 'darkorange', 'tomato', 'orangered', 'red', 'crimson',
		  'salmon', 'coral', 'firebrick', 'brown', 'darkred', 'tan', 'rosybrown', 'sienna', 'saddlebrown'
		  ]
	color_display = ['[COLOR=%s]%s[/COLOR]' % (i, i.capitalize()) for i in color_chart]
	if no_color:
		color_chart.insert(0, 'No Color')
		color_display.insert(0, 'No Color')
	choice = selection_dialog(color_display, color_chart, msg_dialog)
	if not choice: return
	return choice
